@@ -1,6 +1,6 @@
 """Handles the running of jobs and, afterward, of the reports."""
 
-# The code below is subject to the license contained in the LICENSE file, which is part of the source code.
+# The code below is subject to the license contained in the LICENSE.md file, which is part of the source code.
 
 from __future__ import annotations
 
@@ -9,19 +9,20 @@ import subprocess
 import sys
 import time
 import traceback
-from concurrent.futures import Future
-from pathlib import Path
-from types import TracebackType
 from typing import TYPE_CHECKING, Any, ContextManager, Iterator, Literal, NamedTuple, TypedDict
 from zoneinfo import ZoneInfo
 
-from webchanges.differs import DifferBase
+from webchanges.differs import DifferBase, ReportKind
 from webchanges.filters import FilterBase
 from webchanges.jobs import NotModifiedError
 from webchanges.reporters import ReporterBase
 
 # https://stackoverflow.com/questions/39740632
 if TYPE_CHECKING:
+    from concurrent.futures import Future
+    from pathlib import Path
+    from types import TracebackType
+
     from webchanges.jobs import JobBase
     from webchanges.main import Urlwatch
     from webchanges.storage import SsdbStorage, _Config, _ConfigDifferDefaults
@@ -66,7 +67,7 @@ class JobState(ContextManager):
     _http_client_used: str | None = None
     error_ignored: bool | str
     exception: Exception | None = None
-    generated_diff: dict[Literal['text', 'markdown', 'html'], str]
+    generated_diff: dict[ReportKind, str]
     history_dic_snapshots: dict[str | bytes, Snapshot]
     new_data: str | bytes = ''
     new_error_data: ErrorData = {}
@@ -88,12 +89,11 @@ class JobState(ContextManager):
     old_timestamp: float = 1605147837.511478  # initialized to the first release of webchanges!
     traceback: str
     tries: int = 0  # if >1, an error; value is the consecutive number of runs leading to an error
-    unfiltered_diff: dict[Literal['text', 'markdown', 'html'], str]
+    unfiltered_diff: dict[ReportKind, str]
     verb: Verb
 
     def __init__(self, snapshots_db: SsdbStorage, job: JobBase) -> None:
-        """
-        Initializes the class
+        """Initializes the class
 
         :param snapshots_db: The SsdbStorage object with the snapshot database methods.
         :param job: A JobBase object with the job information.
@@ -161,7 +161,7 @@ class JobState(ContextManager):
         """Loads form the database the last snapshot(s) for the job."""
         guid = self.job.guid
         self.old_snapshot = self.snapshots_db.load(guid)
-        # TODO Remove these
+        # TODO: Remove these
         (
             self.old_data,
             self.old_timestamp,
@@ -227,7 +227,7 @@ class JobState(ContextManager):
                 data, self.new_etag, mime_type = self.job.retrieve(self, headless)
                 logger.debug(
                     f'Job {self.job.index_number}: Retrieved data '
-                    f'{dict(data=data, etag=self.new_etag, mime_type=mime_type)}'
+                    f'{{data=data, etag=self.new_etag, mime_type=mime_type}}'
                 )
 
             except Exception as e:
@@ -298,7 +298,7 @@ class JobState(ContextManager):
 
     def get_diff(
         self,
-        report_kind: Literal['text', 'markdown', 'html'] = 'text',
+        report_kind: ReportKind = 'plain',
         differ: dict[str, Any] | None = None,
         differ_defaults: _ConfigDifferDefaults | None = None,
         tz: ZoneInfo | None = None,
@@ -348,10 +348,7 @@ class Report:
     start: float = time.perf_counter()
 
     def __init__(self, urlwatch: Urlwatch) -> None:
-        """
-
-        :param urlwatch: The Urlwatch object with the program configuration information.
-        """
+        """:param urlwatch: The Urlwatch object with the program configuration information."""
         self.config: _Config = urlwatch.config_storage.config
         self.tz = (
             ZoneInfo(self.config['report']['tz'])
@@ -472,14 +469,11 @@ class Report:
             if job_state.verb == 'repeated_error' and job_state.job.suppress_repeated_errors:
                 return True
             # Skip empty diffs unless empty-diff is configured
-            if (
+            return (
                 job_state.verb == 'changed'
                 and not self.config['display']['empty-diff']
                 and job_state.get_diff(tz=self.tz, differ_defaults=self.config['differ_defaults']) == ''
-            ):
-                return True
-
-            return False
+            )
 
         for job_state in job_states:
             if not should_skip_job(self, job_state):
